//***********************************//
//      Processing Interpretor       //
//             ESP32  card           //
//           Curve Simulator         //
//***********************************//
//        Christian Couderc          //
//         www.voilec.com            //
//***********************************//
//   220108
// Todo:  Sun library / Change sensor / Get real values in place of simulator
//https://stackoverflow.com/questions/35282540/size-function-not-working-in-processing-3-0

#define DS3231 // Comment if RTC not present !

#include "C_Interpret.h"

#ifdef DS3231
#include "RTClib.h" // Adafruit 
RTC_DS3231 rtc; 
#else
#include <Wire.h>  // included in ds3231.h 
#endif

const int xSize = 900 ; // 160+720+20
const int ySize = 800 ; // 680+100+20
const int leftMargin = 160 ;
const int upperMargin = 70 ;
const int rightMargin = 20 ;
const int lowerMargin = 50 ;

const int NumberOfpoints = 720;
int bufEep720 [NumberOfpoints];
    
#define colBcg 0, 0, 0   // background color = Black

#define colHorizLinesitplabels 102, 255, 51 // = light Green
#define fontHlinesSize 24
// beware: month 1 to 12 !
const char*  DAYWEEK[] = {"Dimanche", " Lundi  "," Mardi  ","Mercredi"," Jeudi  ","Vendredi"," Samedi "};  // 8 car
const char*  MONTH[] = {"Janvier ","Fevrier ","Mars  ","Avril  ","Mai  ","Juin  ","Juillet ","Aout  ","Septembre","Octobre","Novembre","Decembre"};  // 3 a 9
#define sensorCount 21
const char* SENSORitplabel [sensorCount] = {"0?","Temp int","Humidity int","QNH int", "104", "RTC", "Tension Bat", "QNH ext" , "Temp ext", "Humidity ext",
 "Solar N", "111", "Solar W", "Tension Bat", "114", "115", "116", "117", "118", "119", "Delta QNH"};
const char*  UNITitplabel [sensorCount] = {"0?","deg C","   %","Bme280 hPa", "204", "deg C", "Volts", "Bme280  hPa", "deg C", "   %",
 "Lux", "211", "Lux", "Volts", "214", "215", "216", "217", "218", "219", "Bme280 hPa"};
char positions[] = {8, 9, 10, 26, 27};


const String town    = "Marseille"; // Sun rise & set calculations
const int   altitude =  238 ; //
const float latitude =  43.355949 ; // North  43°21'21'390
const float longitude =  5.407019 ; // Est  5°24'25.293
const float zenith   = 90+(35/60) ; //Zénith en tenant compte de la réfraction moyenne de l'atmostphère (environ 35'
//int timeOffset = +1 ; // +2 GMT summer // +1 Winter


const int led_pin = 2;    // Initializing the LED pin
const int buzzer  = 13;    // #define GPIO 13 Small loudspeaker (Violet) 
int numWeek, numDay, numMonth, numYear ;
int sensorRank, nbStepsY, stepOff, offMinD, vari, offs, nowTime, nowMinute, nowHour  ;
String riseTime, setTime ;
int hRise, mRise, hSet, mSet;
int stateMouse, xMouse, yMouse;
char buffer [80];

C_Interpret itp;
//*************************************

void setup () 
{
pinMode(led_pin, OUTPUT); 
pinMode(buzzer, OUTPUT);
Serial.begin(115200); 
#ifdef DS3231
  rtc.begin() ;
#endif
 
delay (100) ; // Wait sending boot informations
ifRTC() ;
pinPon (2) ;
introDemos () ; // For commands checking only, select actions
} 

//*************************************

void loop() 
{ 
createDemoCurve ();
staticFrame () ;
placeMarks () ;
displayCurve () ;
loopWaiting ();
}

//*************************************

void createDemoCurve ()  // Simulation metier
{
getTimeNow ();

sensorRank = 1 + random (sensorCount-1) ;
nbStepsY = 3+random (7) ;
stepOff = 1+ random (3); // 0, 1, 2
int listStep []= {10,25,50} ;
stepOff = listStep [stepOff] ; // 10,25,50
offMinD =  stepOff * random (100) ; // For itplabel printing on Y
offMinD = offMinD - (offMinD%stepOff) ;

/*Serial.print ("offMinD = ") ;
Serial.print (offMinD) ;
Serial.print (" \tstepOff = ") ;
Serial.print (stepOff) ;
Serial.print (" \tnb Lines Y = ") ;
Serial.print (1+nbStepsY) ;*/

int Ys= ySize - upperMargin - lowerMargin  ;
for (int i = NumberOfpoints-1 ; i >= 0 ; i--)   // Brute values     
    {
    vari =  random (90);
    offs = offs + random (vari) - random (vari) ; 
        while (offs > Ys)
            offs = offs - random (vari) ;  
      
        while (offs <0)
            offs = offs + random (vari)  ;       
          
    bufEep720[i] =  Ys-offs ; // Alea  
    }
}

//*************************************

void staticFrame ()  // Curve area
{
#define colPlotAreaDay 255, 255, 230 // Curve zone day = light yellow
#define colPlotAreaNight 191, 191, 191 // Curve zone night = grey
#define WidthBorderArea 2   // Width of lines around plot area  
delay (1000);
itp.background (colBcg) ;  
itp.strokeWeight (WidthBorderArea) ; 
//itp.noStroke (); // Disable stroke (outline) 
//itp.colorLines colPlotAreaDay ;
//itp.colorTextFrames (colPlotBorder) ; 
computeSunTime(hRise, mRise, hSet, mSet);
int yUp = upperMargin;
int yDown = ySize-lowerMargin;
int timeRise = (60 * hRise) + mRise ; // levant
int timeSet  = (60 * hSet)  + mSet  ; // Couchant
int xPosRise =  leftMargin-2 + (NumberOfpoints + (timeRise - nowTime)/ 2) %NumberOfpoints ;
int xPosSet  =  leftMargin-2 + (NumberOfpoints + (timeSet - nowTime)/ 2) %NumberOfpoints  ;
 
if ((nowTime < timeRise)  || (nowTime > timeSet))
  { // It's night
/*  itp.colorTextFrames (255,255,255) ;
  itp.labelText ("night", 10, 350); // Debug
    itp.labelNum (nowTime, 10, 150); // Debug
    itp.labelNum (timeRise, 10, 250); // Debug*/
  itp.colorTextFrames (colPlotAreaNight) ;
  itp.rectangleCorners (leftMargin, yUp , xPosRise, yDown) ; // 1
  itp.rectangleCorners (xPosSet, yUp , xSize-rightMargin, yDown) ; // 3
  itp.colorTextFrames (colPlotAreaDay) ;
  itp.rectangleCorners (xPosRise, yUp , xPosSet, yDown) ; // 2
  }
else
  { // It's day
 /* itp.colorTextFrames (255,255,255) ;
  itp.labelText ("day", 10, 50); // Debug  
  itp.labelNum (nowTime, 10, 150); // Debug
    itp.labelNum (timeRise, 10, 250); // Debug*/
  itp.colorTextFrames (colPlotAreaDay) ;
  itp.rectangleCorners (leftMargin, yUp , xPosSet, yDown) ; // 1
  itp.rectangleCorners (xPosRise, yUp , xSize-rightMargin, yDown) ; // 3
  itp.colorTextFrames (colPlotAreaNight) ;
  itp.rectangleCorners (xPosSet, yUp , xPosRise, yDown) ; // 2
  } 
}

//*************************************

void placeMarks ()
{
delay (500);
hLines () ;
timeLines() ;
placeSensorName ();
placeUnitName ();
placeSun ();
}

//*************************************

void hLines () 
{
int baseitplabel = 1000 - random (2000) ;
int itplabelStep = random (baseitplabel) ;
itp.strokeWeight(2);  
itp.textSize (fontHlinesSize) ;
itp.colorTextFrames (colHorizLinesitplabels); // Horizontal lines & itplabel color = light Green
itp.colorLines (0,255,0); // color to draw horizontal lines = Green
for (int lp=0; lp<nbStepsY+1; lp++)
  {
  int posY =   upperMargin+lp*(ySize-upperMargin-lowerMargin)/nbStepsY  ;
  if ((lp !=0)&&(lp!=nbStepsY))
      itp.segment (leftMargin, posY, xSize-rightMargin, posY) ; // Horizontal line 
  itp.labelNum(baseitplabel-(lp*itplabelStep) , leftMargin-80, posY+5);
  } 
} 

//*************************************

void timeLines () 
{
#define colVertiLinesitplabelsEven 204, 204, 255 // light Blue
#define colVertiLinesitplabelsOdd 179, 209, 255
#define colVertiLinesMidnight 64, 64, 64  // Grey  
#define colVertiLinesMidday 255, 51, 0  // Orange
placeClock (xSize-45, 20, 32) ;
placeMainDate () ;
placePosition () ;
int offset= leftMargin - nowMinute/2;
itp.colorTextFrames (77, 148, 255); // Hours light blue
for (int lp = 1; lp<25; lp++)
  {
  int posX =   offset + (lp*NumberOfpoints/24) ;
  int markH = (lp+nowHour)%24 ;  
  itp.textSize (24) ;
  if (markH !=0)
      {
      if (markH%2 == 0)
        {
        itp.strokeWeight(3);  // Width
        if (markH != 12)
            itp.colorLines (colVertiLinesitplabelsOdd) ; 
        else
            itp.colorLines (colVertiLinesMidday) ;       
        }
      else
        {
        itp.strokeWeight(2);  // Width        
        itp.colorLines (colVertiLinesitplabelsEven) ; 
        }
      }  
  else
        {
        itp.strokeWeight(4);  // Width of lines Thicker = 16
        itp.colorLines (colVertiLinesMidnight) ; // Midnight = Grey
        }      
      itp.segment (posX, ySize-lowerMargin, posX, upperMargin) ; 
      
  if (markH%2 == 0)
    {
    if (markH<10)
        itp.labelNum(markH , posX-6, ySize-lowerMargin+22);
    else
        itp.labelNum(markH , posX-11, ySize-lowerMargin+22);
    }
  }
}

//*************************************

void placeClock (int posX, int posY, int fontSizeClock)  // left upper corner position
{
#define colorBackgroundClock 0, 77, 26        // Color to fill shapes : dark Green 
#define colorDigitsClock    102, 255, 51 // Digits color : Green
itp.textSize (fontSizeClock) ;
itp.noStroke (); // Disable stroke (outline) 
itp.colorTextFrames (colorBackgroundClock); 
#define xCornerNW posX-1.2 *fontSizeClock
#define yCornerNW posY-.4*fontSizeClock
#define xCornerSE posX+1.2*fontSizeClock
#define yCornerSE posY+.4*fontSizeClock
//itp.rectangleCorners (posX-1.2 *fontSizeClock, posY-.4*fontSizeClock, posX+1.2*fontSizeClock,posY+.4*fontSizeClock) ; // cartouche
itp.rectangleCorners (xCornerNW, yCornerNW, xCornerSE, yCornerSE) ; // cartouche
itp.colorTextFrames (colorDigitsClock); 
sprintf (buffer,"%02d:%02d", nowHour, nowMinute);
itp.labelText (buffer, posX-1.11*fontSizeClock, posY+.31*fontSizeClock);

/* <<<<<<<<<<<<<  debug
itp.colorLines (255,0,0);
itp.segment (posX,posY-40, posX,posY+40);
itp.segment (posX-40,posY, posX+40,posY);
itp.segment (xCornerNW,yCornerNW, xCornerSE,yCornerSE);
itp.segment (xCornerNW,yCornerSE, xCornerSE,yCornerNW);
 >>>>>>>>>>>>>  debug         */
} 

//*************************************

void placeMainDate ()  // Curve panel
{
#define colMainDate 170, 128, 255 // Violet
#define xMainDate 450
#define yMainDate 25 
itp.textSize (24) ;
itp.colorTextFrames (colMainDate) ; // Text color : Violet  
placeDate (xMainDate, yMainDate) ; 
}

//*************************************

void placeAdjustDate ()  // RTC adjustment panel
{
#define xAdjustDate  80 
#define yAdjustDate  285
itp.noStroke();
itp.colorTextFrames(230, 255, 179); // Background
itp.rectangleCorners(xAdjustDate, yAdjustDate-49, xSize-1, yAdjustDate+8) ;
itp.textSize (64) ; 
itp.colorTextFrames(0, 0, 0); // Bk
placeDate (xAdjustDate, yAdjustDate) ;
}

//*************************************

void placeDate (int posX, int posY)  // 
{ 
getTimeNow () ; 
sprintf (buffer,"%s %02d %s %04d", DAYWEEK[numWeek%7], numDay, MONTH[numMonth%12], numYear);
itp.labelText (buffer, posX, posY);
} 

//*************************************

void placePosition ()  // 
{
#define colPosition 153, 153, 102 // brown
#define xPosition 450 
#define yPosition 52 
itp.textSize (16) ;
//itp.noStroke (); // Disable stroke (outline) 
itp.colorTextFrames (colPosition) ; 
sprintf (buffer,"%s       Lat:%f    Long:%f    Alti:%03d", town, latitude, longitude, altitude);
itp.labelText (buffer, xPosition, yPosition);
} 

//*************************************

void displayCurve ()
{
#define widthCurve 2 
#define colCurve 0,0,255 // Blue  
/*itp.strokeWeight(2);  // Width of lines Thicker = 16  
itp.colorLines (0,0,255); // Curve color*/
itp.strokeWeight (widthCurve) ; 
itp.colorLines (colCurve) ; 
//startingitp.point (xSize - rightMargin , bufEep720[NumberOfpoints-1]) ;
int x0= xSize - rightMargin  ;
int y0= bufEep720[NumberOfpoints-1] ;
for (int lp = NumberOfpoints-1 ; lp >= 0 ; lp--)      
    {
    itp.segment (x0, y0, lp+leftMargin,upperMargin+ bufEep720[lp]) ; // posX, posY
    x0= lp+leftMargin  ;
    y0= upperMargin+ bufEep720[lp] ;
//startingitp.point (lp+leftMargin,upperMargin+ bufEep720[lp]) ;
    }
} 

//*************************************

void placeFrame ()
{ 
//sendCommand (55) ; //itp.noFill Disable filling geometry
//itp.colorTextFrames (204, 102, 0); // color to fill frames and text
//sendCommand (9,leftMargin-2,upperMargin-2, NumberOfpoints+4, ySize-upperMargin-lowerMargin+4 ) ; // frame

itp.colorTextFrames (102, 255, 51); // color to fill frames and text = light Green
itp.colorLines (0,0,150); //RGB colors for horizontal lines
for (int lp= 1; lp<nbStepsY; lp++)
  itp.segment (xSize-rightMargin, upperMargin+lp*(ySize-upperMargin-lowerMargin)/nbStepsY, leftMargin, upperMargin+lp*(ySize-upperMargin-lowerMargin)/nbStepsY) ; // horizontal line 
}

//*************************************

void placeSensorName ()
{
#define colSensorName 179, 179, 255
#define fontSensorSize 24
#define xSensorName 5
#define ySensorName 20
itp.textSize (fontSensorSize) ;
itp.colorTextFrames (colSensorName) ; // color to fill frames and text = light Brown
String itplabel = SENSORitplabel [sensorRank]; // sensorRank
itp.labelText (itplabel, xSensorName, ySensorName);
}

//*************************************

void placeUnitName ()
{
#define colUnitName colHorizLinesitplabels
#define fontUnitName 24
#define xUnitName leftMargin-15 
#define yUnitName upperMargin-7
itp.textSize (fontUnitName) ;
itp.colorTextFrames (colUnitName); // color to fill frames and text = light Green
String itplabel = UNITitplabel [sensorRank]; // fontUnitName
itp.labelText (itplabel, xUnitName, yUnitName);
}

//*************************************

void placeSun () //(int tRise, int tSset)
{
#define colSunTime 255, 204, 0 // Orange
#define fontSunTime 18
#define xRise 275
#define yRise 25
#define xSet  xRise
#define ySet  52  
itp.textSize (fontSunTime) ;
itp.colorTextFrames (colSunTime); // Orange
sprintf (buffer,"Sun rise = %02d:%02d", hRise, mRise);
itp.labelText (buffer, xRise, yRise);
sprintf (buffer,"Sun set  = %02d:%02d", hSet, mSet);
itp.labelText (buffer, xSet, ySet);
itp.placeImage ("../solair.gif", xSet-40, yRise-10, xSize, ySize );
}

//*************************************

void computeSunTime(int &hRise, int &mRise, int &hSet, int &mSet) // Compute setTime & riseTime
{
hRise = 6 + random (2);
mRise= random (60);
hSet = 18 + random (2);
mSet=  random (60);
}

//*************************************

int checkMouse() 
{
stateMouse = 0 ;
if (Serial.available ( ) > 0) // Checking if the Processing IDE has send a value or not
  {   
//  itp.textSize (120) ;
//  itp.colorTextFrames (0,0,0); // Black
  
  Serial.read(); 
  stateMouse= Serial.readStringUntil(',').toInt();  
  Serial.read(); 
  xMouse = Serial.readStringUntil(',').toInt();
  Serial.read(); 
  yMouse = Serial.readStringUntil('\n').toInt();  

 if (stateMouse==1)
        beep (880,200) ; // La 4
   else     
        beep (220,200) ; // La 2
  }    
return (stateMouse);
}

//*************************************

void mainPanelInterpreter () // Data interpretation on main panel : Exemple = clock adjust, change sensor,...
{
if ((yMouse<35)&&(xMouse> xSize-150)) // right top of screen
      clockAdjust ();
else if ((yMouse<50)&&(xMouse<150)) // left top of screen
      sensorChange();
else
      showsMousePosition ();   // Anywhere
}
  
//*************************************

void sensorChange()
{
itp.colorTextFrames(230, 0, 0) ;
itp.textSize (64) ;
itp.background (255, 204, 230); // violet
itp.labelText ("Change Sensor", 200,200);  
itp.labelText ("Page to be completed !", 10,300);  
}

//*************************************

void showsMousePosition ()
{
String button; 
if(stateMouse == 1)             
    {
    itp.background (255, 204, 153); // Light Orange
    button = ("LEFT");
    }  
else if (stateMouse == 2) 
    { 
    itp.background (128, 212, 255); // light Blue
    button = ("RIGHT");
    } 

itp.colorTextFrames(255, 0, 0) ;  //Red
itp.textSize (100) ;
itp.labelText ("button :", 150,100); 
itp.labelText (button, 550,100); 

itp.colorLines (0, 0, 0) ; // Black
itp.segment (xMouse, 0,  xMouse, ySize);
itp.segment (0 , yMouse, xSize, yMouse); 

itp.colorTextFrames (26, 26, 255) ; // Medium Blue
itp.textSize (64) ;
itp.labelText("Mouse X =", 10,650);
itp.labelText("Mouse Y =", 10,750);
itp.labelNum(xMouse, 300,650);
itp.labelNum (yMouse, 300,750);

delay (2000);
}

//*************************************
