/////////////////////////////////////////////////////////// 
//////////                                     	///////////
////////// 	       	Hardware for board :    	///////////
//////////          Due clones + ili9340        ///////////
//////////               480x320                ///////////
//////////          Version       180930       	///////////
//////////          Christian    Couderc        ///////////
//////////          http://www.voilec.com       ///////////
//////////                                      ///////////
/////////////////////////////////////////////////////////// 

/*
Nextion NX4832T035 3.5 480x320 HMI TFT LCD
* 
Afficheur graphique ili9341 480x320
Encodeur  KY040
BMP280 Capteur pression
HP
One Wire + Dallas DS18b20 Capteur température
Horloge temps réel   DS3231 +AT24C512
Switches 49, 51
****** Options : 
Potentiomètre    270 °   10 kOhms
Potentiomètre 10 tours 10 kOhms
Switches N,S,E,W
*/

#ifndef _CC_colors 
#define _CC_Colors
#include <CC_Colors.h>
#endif

const char boardName [] = "Due_Clones" ; // 10 caracters

//#define dark


#define line3Space  47  	// car 3 pour HX8357

#define EEP24C512 		0x57  	 // External EEProm 64 ko > (A2 nc) A1 A0 : Ground 0x50 , VCC 0x57 with DS3231
#define EEPROM_Size		0x10000	 //   65536 

/////////////////////////////////////////////////////////////////// 

// Pins 0,1 not avalaible (Rx Tx  reserved for serial 0 USB)

#define Rx		0
#define Tx		0

// <<<<<<<<<<<<<<<  Lignes bloquées par l'afficheur

// Bloque toutes les lignes d'alim mais n'utilise que Gnd et 3.3VC

#define LCD_RST  A4  // Reset pin (could connect to Arduino RESET pin)
#define LCD_CS 	 A3  // Chip select control pin
#define LCD_RS 	 A2  // TFT_DC  48  // Data Command control pin ? 
#define LCD_WR 	 A1 
#define LCD_RD 	 A0 

// VIN
// GND
// GND
//   5 V  : Only one is connected !
// 3.3 V  : Only one is connected !
// RST 

#define LCD_D0 	  8 
#define LCD_D1 	  9 
#define LCD_D2 	  2 
#define LCD_D3    3 
#define LCD_D4 	  4 
#define LCD_D5 	  5 
#define LCD_D6 	  6 
#define LCD_D7 	  7 
 
#define SD_SS	 10  // SD card not working !!
#define SD_DI	 11 
#define SD_DO	 12 
#define SD_CLK	 13  // Twin use

// 32k square Wave output of DS3231 not connected

//#define Joystick_Switch	 47  // mini joystick switch
//#define Joystick_NS	 	 A5  // analog joystick Nord South
//#define Joystick_EW	 	 A6  // analog joystick East West

//  ///////////////////////   A définir ultérieurement 

// A7

#define redPot			A8
#define greenPot		A9
#define bluePot			A10
#define SupplyVolt		A11

// ///////////////////////////////////

// Adresses de base par straps (modifiables) Couleurs à définir

#define Led0			 0	// Rx
#define Led1		 	 1	// Tx
//  2 .. 12   >>>>>>>> Used by display ili9341 480x320
#define LedCard			13  // Yellow
// 14 .. 19
// 20 SDA
// 21 SCL
#define BackLight 		22  // Gate Mosfet retrolight
// 23
#define Buzzer			24  //	Small loudspeaker
//  25 .. 33
#define OrderHeat 		34  // Gate Mosfet Heating
// 35
#define Blackout		36  // 0 = no display, no sound, info from ATTiny
// 37
#define Led_0 	 		38 	// Red >>>> Leds Physical positions
// 39
#define Led_1 	  		40	// Green
// 41
#define Led_2 	  		42	// Blue
// 43
#define Led_3     		44	// Yellow
#define clockIntPin   	45 	// DS3231 by wrapping 5 of JP7 (14 of 20 pins plug)
#// 32k square Wave output of DS3231 not connected
#define Led_4 	  		46	// White
#define DallasTempPin 	47 	// by wrapping 4 of JP7 (12 of 20 pins plug)
#define Led_5 	  		48	// Red
#define Push_49	  		49	// Interrupt 49
#define KY040_Push    	50 	// Switch encoder (pullup) 
#define Push_51	  		51	// Interrupt 51
#define KY040_Data    	52 	// 
#define KY040_Clock   	53 	//

#define LedR	Led_0 
#define LedG	Led_1 
#define LedB	Led_2 
#define LedY	Led_3 
#define LedW	Led_4 
#define Led2R	Led_5 

const int ListLeds [] = {Led_0, Led_1, Led_2, Led_3, Led_4, Led_5} ;
const byte nbLeds = (sizeof (ListLeds)) / 4 ;
// nbLeds = (sizeof (ListLeds)) / 4 = 6

// ///////////////////////////////////

// nbChannels = (sizeof (sensors))/8) = 5       Channel 0 = no device
#define nbChannels 5

// First channel identified by discoverOneWireDevices ()
byte sensors [nbChannels][8] = 
{
{0x28 , 0x44 , 0xC3 , 0xC7 , 0x05 , 0x00 , 0x00 , 0x51} , 	/* Sensors 1 Nord 3 */
{0x28 , 0x3C , 0x3B , 0xC8 , 0x05 , 0x00 , 0x00 , 0x6A} , 	/* Sensors 2 Salon 1 */
{0x28 , 0xFF , 0x8F , 0x75 , 0xB2 , 0x15 , 0x03 , 0xC7} ,	/* Sensors 3 Voyage 1 */
{0x28 , 0xFF , 0x27 , 0x98 , 0xB2 , 0x15 , 0x03 , 0x1E} , 	/* Sensors 4 Spare x */
{0x28 , 0xFF , 0x20 , 0x5A , 0xB2 , 0x15 , 0x03 , 0x71}  }; /* Sensors 5 Spare x */


byte  addr1_1[] = {0x28 , 0x5C , 0xA3 , 0x08 , 0x00 , 0x00 , 0x00 , 0x2C} ; // Salon 0 ancien uno
byte  addr2_1[] = {0x28 , 0xFF , 0xA8 , 0xC7 , 0x05 , 0x00 , 0x00 , 0xCF} ; // Chaudiere 1
byte  addr3_1[] = {0x28 , 0x44 , 0xC3 , 0xC7 , 0x05 , 0x00 , 0x00 , 0x51} ; // Nord

byte  addr1_2[] = {0x28 , 0x3C , 0x3B , 0xC8 , 0x05 , 0x00 , 0x00 , 0x6A} ; // Salon 1
byte  addr2_2[] = {0x28 , 0x03 , 0xB1 , 0xC6 , 0x05 , 0x00 , 0x00 , 0xC0} ; // Nord 2
byte  addr3_2[] = {0x28 , 0x52 , 0x88 , 0xC7 , 0x05 , 0x00 , 0x00 , 0x21} ; // Chaudière 3

byte  addr1_3[] = {0x28 , 0xFF , 0x8F , 0x75 , 0xB2 , 0x15 , 0x03 , 0xC7} ; // Only one in this case :  Voyage
byte  addr2_3[] = {0x28 , 0xFF , 0x8F , 0x75 , 0xB2 , 0x15 , 0x03 , 0xC7} ;
byte  addr3_3[] = {0x28 , 0xFF , 0x8F , 0x75 , 0xB2 , 0x15 , 0x03 , 0xC7} ;

byte  addr1_4[] = {0x28 , 0xFF , 0x27 , 0x98 , 0xB2 , 0x15 , 0x03 , 0x1E} ; // Only one in this case : Spare
byte  addr2_4[] = {0x28 , 0xFF , 0x27 , 0x98 , 0xB2 , 0x15 , 0x03 , 0x1E} ;
byte  addr3_4[] = {0x28 , 0xFF , 0x27 , 0x98 , 0xB2 , 0x15 , 0x03 , 0x1E} ;

byte  addr1_5[] = {0x28 , 0xFF , 0x20 , 0x5A , 0xB2 , 0x15 , 0x03 , 0x71} ; // Only one in this case : Spare
byte  addr2_5[] = {0x28 , 0xFF , 0x20 , 0x5A , 0xB2 , 0x15 , 0x03 , 0x71} ;
byte  addr3_5[] = {0x28 , 0xFF , 0x20 , 0x5A , 0xB2 , 0x15 , 0x03 , 0x71} ; 

// channel 0 dummy, not exist
int8_t   coefPressAdd   [6] = {     0,     0,     0,      0,     0,     0} ; // +/- 127 Pascals
uint16_t coefPressMult  [6] = {     0,  9919,  9918,   9921,  9920,     0} ; // 10k factor
int8_t   coefSalonAdd   [6] = {     0,    79,    35,      0,     0,     0} ; // +/- 127/100 degrés 
// coefSalonTempMult
int8_t   coefNordAdd    [6] = {     0,     0,     0,      0,     0,     0} ; // +/- 127 /100 degrés
// coefNordTempMult
int8_t   coefBureauAdd  [6] = {     0,     0,     0,      0,     0,     0} ; // +/- 127 /100 degrés
// coefBureauTempMult
int8_t   coefBmp280Add  [6] = {     0,     0,     0,      0,     0,     0} ; // +/- 127 /100 degrés
// coefBmp280TempMult


float CoefV3Mult = 0.13291 ; // Voltmeter adjustment channel 3
float CoefV3Add = 1049.81 ; //

float CoefV4Mult = 0.13291 ; // Voltmeter adjustment channel 4
float CoefV4Add = 1049.81 ; //

/////////////////////////////////////////////////////////////////// 


///////////////  Common parameters for clock DS3231   ///////////////////////

const char*  DAY[] = {"Dimanche", " Lundi  "," Mardi  ","Mercredi"," Jeudi  ","Vendredi"," Samedi "};  // 8 car
const char*  MON[] = {"Janvier ","Fevrier ","Mars  ","Avril  ","Mai  ","Juin  ","Juillet ","Aout  ","Septembre","Octobre","Novembre","Decembre"};  // 3 a 9
const int  QUANT[] = {0,0,31,59,90,120,151,181,212,243,273,304,334} ;

/////////////////////////////////////////////////////////////////// 


