
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    // ARDUINO BATTERY CAPACITY TESTER
    //Version-2.0   https://www.instructables.com/DIY-Arduino-Battery-Capacity-Tester-V20/
    //by deba168,INDIA // The code is taken from Hesam Moshiri ( https://www.pcbway.com/blog/technology/Battery_capacity_measurement_using_Arduino.html )
    //Dated : 20/10/2019
    //
    //Dated : 29/02/2020
    //Added code to read a band gap reference connected to analog pin A1,
    // thus allowing measurement of Vcc and avoiding the need to measure it using a multimeter
    //Replaced 1 ohm 5% resistor with 1 ohm 1% power resistor
    //Modified code to compute actual current draw based on desired values. Measurements
    // using ammeter show computed values within 2 milliamps of measured
    //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    
//220123 Version-3.03 fork by Christian Couderc  https://www.voilec.com

// This original work is excellent, but needs some minor adjustments
// Adapted for dual colors (first line yellow) displays and new texts placements
// Adapted for loud speaker in place of buzzer, various sounds
// Acuracy of current is about 1 for 1000
//Various bugs corrected
// Important hardware modifications :
// 1) Cut input of supply jack --> 12 V supply (not critical, 8 to 16) on Vin nano AND pin 8 LM358 (in place of original 5V removed)
// (LM358 is not rail to rail and can't output the 4.86 V needed on gate for a 1 A current! (very critical !)
// 2) Take output 5V from Nano to Oled AND 1.2V ref , with 33 kOhms (in place of led near jack)
// 3) Place a wire to send 1.2 V on A1
// 4) up/down buttons resistors not needed, input pullup forced now in instantiation
// Software adustments in first next lines:
// Fine adjust of values of 1.2V and R (about 1 Ohm, exemple 1.012)) in firmware
// Better accuraty of time during dischage


//  Tip to control current : Don’t use a serial ammeter, but read voltage on the 1 Ohm resistor (I=V/R). (ou know exactly the real value about 1 Ohm)

/* It is necessary to use a big radiator because the MosFet is not used classicaly in commutation with its very low RDSon and very low dissipation,  but as a power transitor !
It works as a variable resistor wich gererate a lot of heat.
If the current is adjusted to 1 Amp, on 1 Ohm resistor dissipation is 1 Watt at 1 Volt.
With a battery of 4 V, it is necessery to dissipate a povwer of P=V*I = (4-1) * 1 = 3 Watts, three times more than on the resistor, and the radiator is very hot…
*/
// Some measurements on gate  / Current (variable supply in place of LM358)
//3.59V -> 0.1A /// 3.8V -> 0.2 A /// 4.21V -> .5 A /// 4.86V -> 1A

// Todo on V4.0 : Add a µSdcard support to record discharge curves in Excel format, ready to read on PC

// Todo in V5.0 : Add a graphical display 480*320 to show discharge curve in real time

// Version 3.0 
// ->3.01 : Timing corected in keypress 
// ->3.02: Bug corrected: current before start discharge
// *******************************************************

float Vref_Voltage = 1.242 ; // LM385BLP-1.2 Band Gap Reference voltage -> Need to be exactly adjusted
float Resistance = 1.052; // Value of R3 Power Resistor  -> Need to be exactly adjusted
            
#define DEBUG 4 // Binary Treatment 0=Nothing, 1=Early, 2=Running Voltages, 4=Finish
// 0=Nothing
// 1=Early -> List of parameters
// 2=Running Voltages -> 0:1:19 3.866v 22.1mAh
// 4=Finish 

#include<JC_Button.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
    
// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET 4 // Reset pin # (or -1 if sharing Arduino reset pin)
#define LED_BUILTIN 13 // Led on Nano bord
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
    
void discharge();
void Display_UP_DOWN();
void print_Running_Values();
    
const float Low_BAT_level = 2.80;// Threshold for stopping test
//Desired Current steps with a 3R load (R7)
int Current [] = {0,50,100,200,300,400,500,600,700,800,900,1000};
int PWM [] = {0, 2, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50};
char* mesCur[]= {" Adj Curr"};
int sizeMes =  sizeof(mesCur) ; //=10
int Array_Size;// calculated during setup
const byte PWM_Pin = 10;
const byte buzzer = 9;
const int BAT_Pin = A0;
const int Vref_Pin = A1; //Pin to which Band Gap Reference is attached
int Current_Value = 0; // Selected Current value for test
int PWM_Value = 0;// Value of PWM during test
int PWM_Index = 0;// Index into PWM array during test
unsigned long Capacity;
float Capacity_f;
int ADC_Value = 0;

float Vcc ; // Voltage of Arduino 5V pin ( Computed during setup using Band Gap Reference )
float BAT_Voltage;
float  sample;
int ponderationFactor = 80 ; // Read to understand : https://www.voilec.com/pages/ponder.php
long topTime ;
int old2Sec;
byte Hour = 0, Minute = 0, Second = 0;
char buffer[9];
bool calc = false, Done = false, press = false, Report_Info = true;
// From library : pinMode(m_pin, m_puEnable ? INPUT_PULLUP : INPUT);
Button UP_Button(2, 25, true, true); // No need for external resistor, internal enabled false -> last true
Button Down_Button(3, 25, true, true);
    
// string values for reporting intermediate information
const int VAL_MAX = 10;
char val_0[VAL_MAX]={""};
char val_2[VAL_MAX]={""};
    
// **********************************************
// **********************************************

void setup () 
{    
Serial.begin(115200);
pinMode(PWM_Pin, OUTPUT);
pinMode(buzzer, OUTPUT);
pinMode(LED_BUILTIN, OUTPUT);
analogWrite(PWM_Pin, PWM_Value);
UP_Button.begin();
Down_Button.begin();
while (!Serial);

nStars (12);
Serial.print (F("\nBATTERY CAPACITY TESTER\nVersion 3.01\nCompilation :  ")) ;
// Example of __DATE__ string: "Jul 27 2012"
//                              01234567890  11 car
Serial.print (__DATE__) ; // Compilater Gcc constants
Serial.print (F("     ")) ;
// Example of __TIME__ string: "21:06:19"
//                              01234567  8 car
Serial.println (__TIME__) ; 
Serial.println ("") ; 
nStars (2);

display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
delay (10);
display.clearDisplay();
display.setTextColor(WHITE);
display.setTextSize(1);
display.setCursor(12,2);
display.print(F("Open Green Energy"));
display.setCursor(28,25);
display.print(F("Version 3.01"));
display.setCursor(26,40);
display.print(__DATE__);
display.setCursor(38,55);
display.print(__TIME__);

display.display();
soundSwitchAction (); 
float fTemp=0.0;
//Read voltages of Band Gap Reference and use it to calculate actual Vcc
for(int i=0;i< 1000;i++)  // Make the 3 seconds delay
      {
      fTemp += analogRead(Vref_Pin); //read the Band Gap Reference voltage
      delay (3);
      }
Vcc = Vref_Voltage * 1024000 / fTemp;
      
//testFonts (); // Font1=20 car / 2=10 /3=7    // Optionnal   
   
    #if (DEBUG == 1 || DEBUG == 5)
    Serial.println(F("\nStart of calculations"));
    #endif
    
Array_Size = sizeof(Current)/2;

    
    // Convert desired current levels to PWM using actual Vcc
    // Convert PWM values back to actual current levels
    // While measuring actual current I discovered that the actual draw is
    // (PWM + 1)/256 rather than PWM/255 as indicated in Arduino documentation
int iTemp;
for (int i=0;i<Array_Size;i++)
      {
      iTemp = int( Resistance * Current[i] * 256.0 / Vcc / 1000.0 - 1.0 + 0.5 ); // desired current to nearest PWM
      iTemp = min(iTemp,255);
      iTemp = max(iTemp,0);
      Current[i] = int( (iTemp+1) * Vcc / 256.0 / Resistance * 1000.0); // actual current for PWM
      PWM[i] = iTemp;//Save PWM in array
      }
    
    //Include Threshold and Vcc in startup display
dtostrf(Low_BAT_level, 5, 3, val_0);
dtostrf(Vcc, 5, 3, val_2);
    
display.clearDisplay();
display.setTextSize(1);
display.setCursor(2,3);
display.print("Thr=");
display.print(val_0);
display.print("v  Vcc=");
display.print(val_2);
display.setTextSize(2);
display.setCursor(6,16);
display.print("Adj Curr:");
display.setCursor(2,40);
display.print("UP/Down:");
display.print("0");   
display.display();
sample = analogRead(BAT_Pin); //read the Battery voltage :  Instantiate ponderation filter
topTime=millis();
BAT_Voltage =  batValue (5000) ; // Measuring Battery Voltage and delay in mS
soundReady ();
}   
    
// **********************************************
    
void loop() 
{ 
if (Report_Info)
      {             //Report_Info
//    Serial.flush();
#if (DEBUG == 1 || DEBUG == 5)
    Serial.print(F("Threshold: "));
    Serial.print(Low_BAT_level,3);
    Serial.print(F(" volts\nR3 Resistor: "));
    Serial.print(Resistance,3);
    Serial.print(F(" ohms\nMeasured Vcc Voltage: "));
    Serial.print(Vcc,4);
    Serial.println(F(" volts"));
    
    Serial.println(F("Index Actual(mA) PWM"));
    for (int i=0;i<Array_Size;i++)
      {
      Serial.print("[");
      Serial.print(i);
      Serial.print("]");
      Serial.print(" ");
      Serial.print(Current[i],DEC);
      Serial.print(" ");
      Serial.print(PWM[i],DEC);
      Serial.println(" ");
      }
#endif
    Report_Info = false;
      }         //Report_Info
    
UP_Button.read();
Down_Button.read();
if (UP_Button.wasReleased() && PWM_Index < (Array_Size-1) && calc == false)
      {
      PWM_Value = PWM[++PWM_Index];
      //analogWrite(PWM_Pin,PWM_Value);
      soundRecord () ;
      press = true;
      }
    
if (Down_Button.wasReleased() && PWM_Index > 0 && calc == false)
      {
      PWM_Value = PWM[--PWM_Index];
      //analogWrite(PWM_Pin,PWM_Value);
      soundRecord ();
      press = true;
      }
    
if (UP_Button.pressedFor (1000) && calc == false) // -> Starting discharge
      {
      soundSwitchAction (); 
      display.clearDisplay();
      topTime = millis() ;
      analogWrite(PWM_Pin,PWM_Value);
      discharge();
      }
Display_UP_DOWN();
}
    
//***********************************************
// **********************************************
    
void discharge() // Increment seconds
{
calc = true;
while (Done == false) 
 {//(Done == false)
    Second ++;
    if (Second >59) //== 60) 
      {
      Second = 0;
      Minute ++;
      }
    if (Minute >59) //== 60) 
      {
      Minute = 0;
      Hour ++;
      }
    
BAT_Voltage = batValue (1000) ; // Measuring Battery Voltage and delay of 1 second
digitalWrite (LED_BUILTIN , Second%2) ;
     
display.clearDisplay();
display.setTextSize(2);
display.setCursor(15,1);
dispDuration() ;
    
display.setTextSize(1);
display.setCursor(0,20); //25
display.print(Current_Value);
display.print(" mA");

display.setCursor(50, 20);
display.print(Capacity_f,1);
display.print(" mAh");
display.setTextSize(3);
display.setCursor(8,36);
display.print(BAT_Voltage,3);
display.setCursor(105,40);
display.setTextSize(2);
display.print("V");
    
    // When total seconds is greater than 32767 the statement below did not work until the byte values
    // Hour, Minute and Second were cast to an unsigned long. Apparently the compiler cast the byte
    // values to an "int" first which cannot represent 32768 correctly
Capacity = ((unsigned long)Hour * 3600) + ((unsigned long)Minute * 60) + (unsigned long)Second;
Capacity_f = ((float)Capacity * Current_Value) / 3600.0;
    
display.display();
    
    #if (DEBUG == 4 || DEBUG == 2)
    print_Running_Values();
    #endif

//*** <<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    
if (BAT_Voltage < Low_BAT_level)  // -------> End of discharge
 {        //BAT_Voltage < Low_BAT_level 
    twinSounds() ;
    //soundNotNow() ;
    #if (DEBUG == 4 || DEBUG == 5)
    Serial.println(F("\nCurrent_Value PWM_Value"));
    Serial.print(Current_Value);
    Serial.print(" ");
    Serial.println(PWM_Value);
    Serial.println(F("\nHour Minute Second PWM_Index"));
    HMS (Hour, Minute, Second);
    Serial.print (buffer);
    Serial.print(" ");
    Serial.println(PWM_Index);
    #endif
    
    // When total seconds is greater than 32767 the statement below did not work until the byte values
    // Hour, Minute and Second were cast to an unsigned long. Apparently the compiler cast the byte
    // values to an "int" first which cannot represent 32768 correctly
    Capacity = ((unsigned long)Hour * 3600) + ((unsigned long)Minute * 60) + (unsigned long)Second;
    
    #if (DEBUG == 4 || DEBUG == 5)
    Serial.println(F("Capacity HMS"));
    Serial.println(Capacity);
    #endif
    
    Capacity_f = ((float)Capacity * Current_Value) / 3600.0;
    
    #if (DEBUG == 4 || DEBUG == 5)
    Serial.println(F("Capacity HMS*PWM"));
    Serial.println(Capacity_f,1);
    #endif
    
    Done = true;
    PWM_Value = 0;
    analogWrite(PWM_Pin, PWM_Value);
    soundSwitchAction (); 
    flashingEnd () ;   
  }//BAT_Voltage < Low_BAT_level
 }//(Done == false)  
}// discharge

// **********************************************

void Display_UP_DOWN()
{
BAT_Voltage =  batValue (100) ; // Measuring Battery Voltage and delay in mS
if (old2Sec == (millis()/2000)&& (press ==false)) 
  return;

old2Sec = millis()/2000 ;
digitalWrite (LED_BUILTIN , old2Sec%2) ;
#if DEBUG!=0
Serial.println (BAT_Voltage);
#endif
press= false ;
Current_Value = Current[PWM_Index];
display.clearDisplay();
display.setTextSize(1);    
display.setCursor(4,6);
if (Current_Value<1000) // CC 210106
      display.print(F("Curr:"));
else
      display.print(F("Cur:")); 
display.setCursor(55,0);
display.setTextSize(2);
display.print(Current_Value);
display.setTextSize(1);
display.setCursor(114,6);
display.print(F("mA"));

if (old2Sec%2==0)
  {
  display.setCursor(45,50);
  display.print(F("PWM : "));
  display.print(int(PWM_Value));
  display.setCursor(15,20);
  display.print(F("long press <UP>"));
  display.setCursor(15,30);
  display.print(F("start discharge"));
  }
else
  {
  display.setTextSize(3);
  display.setCursor(8,38);
  display.print(BAT_Voltage,3);
  display.setCursor(115,45);
  display.setTextSize(2);
  display.print("V");
  }
  
display.display();
}
    
// **********************************************
    
void print_Running_Values()
{
HMS (Hour, Minute, Second);
Serial.print (buffer);
Serial.print(F(" "));
Serial.print(BAT_Voltage,3);
Serial.print("v ");
Serial.print(Capacity_f,1);
Serial.println(F("mAh"));
}

// **********************************************
// **********   some sound addons   ************* 
// **********************************************   

void beep (int frequency, int duration)  // duration > 1 ms
{
float interval = 500000/frequency ;  // interval 500 microsec for 1 kHz

long topMicro1= micros () ;
while ((micros () - topMicro1)< (long)duration*1000)
  {
  digitalWrite (buzzer, !digitalRead(buzzer)) ; 
  long topMicro= micros () ;
  while ((micros () - topMicro)< interval)  ;
  }
digitalWrite (buzzer, LOW) ; 
}

// ************************************************************************

void beepP (int frequency, int duration, int pause)
{
beep (frequency, duration) ; 
delay (pause) ; 
}
    
// ************************************************************************

void soundSwitchAction () 
{
for (int i = 0 ; i < 6 ; i++)
  beep (1046 * (1 + 4 * (i % 2)) , 25) ; // 1046 Do 5 // 4186 Do 7
delay (100);
}

// ************************************************************************

void soundRecord () // keypress
{
//beepDurFreq (25,  1000 ) ; // Duration  Frequency 
beepDurFreq (5,  1975 ) ; // Duration  Frequency Si 5
}

// ************************************************************************

void soundReady ()
{
for (int i=20 ; i >0 ; i--)  
    beepDurFreq (25, i*5) ; 
//delay (1000) ;
}

// ************************************************************************

void dispDuration()
{
HMS (Hour, Minute, Second);
display.print (buffer);
}

// ************************************************************************

char* HMS (uint8_t Hour, uint8_t Minute, uint8_t Second)
{
sprintf (buffer, "%02d:%02d:%02d", Hour ,Minute, Second); 
return buffer;
}

// ************************************************************************

void beepDurFreq (int duration, int Frequency)  
{ 
tone (buzzer,  Frequency ); // Example : 440 = La 3
delay(duration); 
noTone(buzzer); 
}

// ************************************************************************

void soundNotNow ()
{
for (int i=500; i<1000; i++) 
  {
  tone(buzzer, i); 
  delay(1); 
  } 
for (int i=1000; i>500; i--) 
  {
  tone(buzzer, i); 
  delay(1); 
  } // fin boucle for 
noTone(buzzer); 
}

// ************************************************************************

void twinSounds ()
{
for (int i=0 ; i<16 ; i++)
    beepDurFreq (100, 220 * (1+16*(i%2))) ;  // 41 Mi 0 // 656 Mi 5
for (int i=15 ; i>0 ; i--)
    beepDurFreq (100, 220 * (1+16*(i%2))) ;  // 41 Mi 0 // 656 Mi 5
noTone(buzzer); 
}

// ************************************************************************
// ***********   end of sound addons  *************************************
// ************************************************************************

void testFonts () // Font1=20 car / 2=10 /3=7
{
display.clearDisplay();
display.setCursor(0,0);
display.setTextSize(1);
display.print ("12345678901234567890"); // test font1 : 20 caractères
display.setCursor(0,17);
display.setTextSize(2);
display.print ("1234567890"); // test font2 : 10 caractères
display.setCursor(0,40);
display.setTextSize(3);
display.print ("1234567"); // test font3 : 7 caractères 
display.display();
delay (4000) ; 
display.clearDisplay(); 
}

// ************************************************************************

void flashingEnd ()
{
int count = 0;
while (1)
  {         // while  -------> Final loop
  display.clearDisplay();

  if (count++%4 ==0)
          {
          display.setTextSize(2);
          display.setCursor(12,0);
          display.print("Capacity");
          display.setCursor(12,43);
          display.print("Duration");
          }
   else    
          {   
          display.setCursor(18,0);
          display.print(int(Capacity_f));
                   // display.print(Capacity_f,1);
          display.print(" mAh");
          display.setCursor(20,35);
          dispDuration();
          }
   display.display();
   
   flashLed (100, 1000) ; // period, duration
   delay (1000);
   }        // while
}

// ************************************************************************

void nStars (int nb)
{
for (int cnt=0; cnt<nb; cnt++)
  {
  for (int c2=nb-cnt; c2>0; c2--)
    Serial.print("* ");
    Serial.println("");
  delay (10);
  }
}

// ************************************************************************

float  batValue (int wait) // Measuring Battery Voltage and delay in mS
{   
while (millis ()-topTime < wait)
    {
    sample=((sample*(ponderationFactor-1)) + analogRead(BAT_Pin)) / ponderationFactor ; //read the Battery voltage
    delay (2);
    }
 
topTime = millis() ;    
return sample * Vcc / 1024.0; //BAT_Voltage;
}

// ************************************************************************

void flashLed (int period, int duration)
{
for (int cnt=0 ; cnt<duration/period ; cnt++)
  {
  digitalWrite (LED_BUILTIN , cnt%2) ;
  delay (period/2);
  }
} 
  
// ************************************************************************
