//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// ARDUINO BATTERY CAPACITY TESTER
// You need an hardware modification and a current of 2 A, using second amplifier, read: 
// https://www.voilec.com/pages/battery_test.php
// *******************************************************
/*
This tool shows PWM value on Oled display, 0..255. You can change it with Up/Down buttons, 
and observe on oscilloscope and voltmeters influence of the various parameters.
For these experiments, replace the 18650 by a power supply 4 V with limit at 3 A.
When this test running, current is active, place a voltmeter on power resistor and check for temperature increase.
*/

float Vref_Voltage = 1.219 ; // LM385BLP-1.2 Band Gap Reference voltage -> Need to be exactly adjusted
float Resistance = 1.004; // Value of R3 Power Resistor  -> Need to be exactly adjusted
            
#define supplyOn true  // Bridge on 12 V-> 2 V on A2, set to false if not


#include<JC_Button.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
    
// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET 4 // Reset pin # (or -1 if sharing Arduino reset pin)
#define LED_BUILTIN 13 // Led on Nano bord
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
    
//Desired Current steps 
//int Current [] = {0,50,100,200,300,400,500,600,700,800,900,1000, 1500, 2000};
//int PWM [] = {0, 2, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 60, 70};


const int Vvolt_Pin = A2; // Pin to read 12V supply
const byte PWM_Pin = 10;
const byte buzzer = 9;

bool calc = false, Done = false, press = false, Report_Info = true;
// From library : pinMode(m_pin, m_puEnable ? INPUT_PULLUP : INPUT);
Button UP_Button(2, 25, true, true); // No need for external resistor, internal enabled false -> last true
Button Down_Button(3, 25, true, true);
 
uint8_t PWM_bk, PWM=60 ;
    
// **********************************************
// **********************************************

void setup () 
{    
Serial.begin(115200);
pinMode(PWM_Pin, OUTPUT);
pinMode(buzzer, OUTPUT);
pinMode(LED_BUILTIN, OUTPUT);
analogWrite(PWM_Pin, 0); // No current on start
UP_Button.begin();
Down_Button.begin();
while (!Serial);

nStars (12);
Serial.print ("\nBATTERY CAPACITY TESTER\nVersion 3.01\nCompilation :  ") ;
// Example of __DATE__ string: "Jul 27 2012"
//                              01234567890  11 car
Serial.print (__DATE__) ; // Compilater Gcc constants
Serial.print ("     ") ;
// Example of __TIME__ string: "21:06:19"
//                              01234567  8 car
Serial.println (__TIME__) ; 
Serial.println ("") ; 
nStars (2);

display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
delay (10);
panel_Intro(); // Compilation informations

soundSwitchAction (); 
delay (2000);
supplySecurity (); // Test if the 12 V supply is present
}   
    
// **********************************************
    
void loop() 
{ 
Serial.print("PWM: ");
Serial.println(PWM);
   
UP_Button.read();
Down_Button.read();

//if (UP_Button.wasReleased() )
if (UP_Button.read() )
      {
      ++PWM;
      soundRecord () ;
      press = true;
      }
    
//if (Down_Button.wasReleased() )
if (Down_Button.read() )
      {
      --PWM;
      soundRecord ();
      press = true;
      }
      
//if (PWM>100)
//    PWM=100;     
analogWrite(PWM_Pin, PWM); // No current on start    

if (PWM !=PWM_bk)
  {
  display.clearDisplay();
  display.setTextSize(1);
  display.setCursor(30,5);
  display.print("PWM = ");
  display.setTextSize(4);
  display.setCursor(40,30);
  display.print(PWM);
  display.display();
  }
}
    
//***********************************************

// **********************************************
// **********   some sound addons   ************* 
// **********************************************   

void beep (int frequency, int duration)  // duration > 1 ms
{
float interval = 500000/frequency ;  // interval 500 microsec for 1 kHz

long topMicro1= micros () ;
while ((micros () - topMicro1)< (long)duration*1000)
  {
  digitalWrite (buzzer, !digitalRead(buzzer)) ; 
  long topMicro= micros () ;
  while ((micros () - topMicro)< interval)  ;
  }
digitalWrite (buzzer, LOW) ; 
}

// ************************************************************************

void beepP (int frequency, int duration, int pause)
{
beep (frequency, duration) ; 
delay (pause) ; 
}
    
// ************************************************************************

void soundSwitchAction () 
{
for (int i = 0 ; i < 6 ; i++)
  beep (1046 * (1 + 4 * (i % 2)) , 25) ; // 1046 Do 5 // 4186 Do 7
delay (100);
}

// ************************************************************************

void soundRecord () // keypress
{
//beepDurFreq (25,  1000 ) ; // Duration  Frequency 
beepDurFreq (5,  1975 ) ; // Duration  Frequency Si 5
}

// ************************************************************************

void soundReady ()
{
for (int i=20 ; i >0 ; i--)  
    beepDurFreq (25, i*5) ; 
//delay (1000) ;
}

// ************************************************************************

void beepDurFreq (int duration, int Frequency)  
{ 
tone (buzzer,  Frequency ); // Example : 440 = La 3
delay(duration); 
noTone(buzzer); 
}

// ************************************************************************

void soundNo12V ()
{
for (int i=50; i<100; i++) 
  {
  tone(buzzer, i); 
  delay(1); 
  } 
for (int i=100; i>50; i--) 
  {
  tone(buzzer, i); 
  delay(1); 
  } // fin boucle for 
noTone(buzzer); 
}

// ************************************************************************

void twinSounds ()
{
for (int i=0 ; i<16 ; i++)
    beepDurFreq (100, 220 * (1+16*(i%2))) ;  // 41 Mi 0 // 656 Mi 5
for (int i=15 ; i>0 ; i--)
    beepDurFreq (100, 220 * (1+16*(i%2))) ;  // 41 Mi 0 // 656 Mi 5
noTone(buzzer); 
}

// ************************************************************************
// ***********   end of sound addons  *************************************
// ************************************************************************

void testFonts () // Font1=20 car / 2=10 /3=7
{
display.clearDisplay();
display.setCursor(0,0);
display.setTextSize(1);
display.print ("12345678901234567890"); // test font1 : 20 caractères
display.setCursor(0,17);
display.setTextSize(2);
display.print ("1234567890"); // test font2 : 10 caractères
display.setCursor(0,40);
display.setTextSize(3);
display.print ("1234567"); // test font3 : 7 caractères 
display.display();
delay (4000) ; 
display.clearDisplay(); 
}

// ************************************************************************

void nStars (int nb)
{
for (int cnt=0; cnt<nb; cnt++)
  {
  for (int c2=nb-cnt; c2>0; c2--)
    Serial.print("* ");
    Serial.println("");
  delay (10);
  }
}

// ************************************************************************

void flashLed (int period, int duration)
{
for (int cnt=0 ; cnt<duration/period ; cnt++)
  {
  digitalWrite (LED_BUILTIN , cnt%2) ;
  delay (period/2);
  }
} 
  
// ************************************************************************

void supplySecurity () // Test it the 12 V supply is present
{
int count;
if (supplyOn == false) // No bridge in place
  return;
 
float  voltage=0;   
#define timeToWait 1000 // Make the 1 second delay
#define smallDelay 3
#define nbLoops timeToWait/smallDelay  // 333
bool voltOn = false;
 
while (voltOn == false)
 {// w
 for(int lp=0; lp< nbLoops; lp++)   
      {
      voltage+= analogRead(Vvolt_Pin); //read the Band Gap Reference voltage
      delay (smallDelay);
      digitalWrite (LED_BUILTIN , lp/20);
      }
voltage /= nbLoops ;
//Serial.print (voltage) ;
voltage *= 0.02758 ; // =  voltage / 1024 * 5.7 * 4.933;
//Serial.print (" \t");
//Serial.println (voltage) ; 
 
if (voltage >8)
  voltOn = true ;
else
  {// e
  soundNo12V() ;
  display.clearDisplay();
  if (count++%4 ==0)
          {
          display.setTextSize(3);
          display.setCursor(10,0);
          display.print("^^^^^");
          display.setCursor(25,16);
          display.print("Plug");
          display.setCursor(25,44);
          display.print("12 V");
          }
   else    
          {   
          display.setTextSize(1);
          display.setCursor(22,0);
          display.print("read source");
          display.setCursor(10,20);
          display.print("for informations");
          display.setCursor(8,43);
          display.print("on bridge 47k/10k");
          }
display.display(); 
    } // e
  } // w    
}

// ************************************************************************

void panel_Intro()
{
display.clearDisplay();
display.setTextColor(WHITE);
display.setTextSize(1);
display.setCursor(0,2);
display.print("Small Battery tester");
display.setCursor(28,17);
display.print("Version 3.2");
display.setCursor(26,30);
display.print(__DATE__);
display.setCursor(38,41);
display.print(__TIME__);
display.setCursor(18,54);
display.print("www.voilec.com");
display.display();
}


// ************************************************************************   
