/*

***********************  
*                     *
*  Christian Couderc  *
*                     * 
*   www.voilec.com    *
*                     *   
***********************  


Algorithme d’optimisation du pas pour le tracé des ordonnées d’une courbe dont les minimums et maximums sont de dimensions quelconques (en flottant).
Utilisation : On passe à la méthode l’amplitude brute = (maximum-minimum), en retour on récupère le pas optimisé qui sera utilisé pour fixer les nouveaux maximums et minimums de la courbe à afficher avec un nombre de marques en ordonnées autour de 6 pour l’ergonomie optimale.
Implantation
 float minimum  = 1234,5678;  // Issu des données collectées
 float maximum  = 56789,01234;  // Issu des données collectées
 float vStep = vStepCalculation (maximum- minimum) ;
Toutes les données à afficher sont normalisées sous la forme :  
<entier> <point décimal><deux chiffres centièmes>
Exemple : 43.21 °C, 12.34 V , 1013.15 hPa…
Mise en œuvre :
 int nouveau_minimum =  minimum- (minimum %vStep ) ;
Affichage : <nouveau_minimum/100><point décimal>< nouveau_minimum/100>
int nouveau_maximum =  maximum - (maximum %vStep) + vStep ; 
Affichage : <nouveau_ maximum /100><point décimal>< maximum /100>
Exemple concret, affichage température
Minimum -4.23°C
Maximum +3.76
Range=  maximum- minimum = 7.99
float vStep = vStepCalculation (maximum- minimum) ; >> résultat 1.50
nouveau_minimum =  minimum- (minimum %vStep ) = -4.23 -1.23 = -3
nouveau_maximum =  maximum - (maximum %vStep) + vStep  = 3.76 -0.76 + 1.50 = 4.50

Ce qui donne sur les échelles optimales de la courbe affichée, occupant le maximum d'espace possible :
<4.50 .. 3 .. 1.50 .. 0 .. -1.5 .. -3>
Soit 6 horizontales chiffre optimal.
Avec un pas de 1, il y aurait eu <4 ..3 .. 2 .. 1 .. 0 .. -1 .. -2 .. -3-.. -4 ..-5>,  soit 10 horizontales ce qui est trop chargé.
Avec un pas de 2, il y aurait eu <4 .. 2 .. 0 ..  -2 .. -4 .. 6> soit aussi 6 horizontales, mais avec beaucoup trop de vide au minimum de la courbe…
*/

void setup() 
{
Serial.begin(115200) ;
while (!Serial) ;
Serial.println("\n\nSTM32 OK\n") ;
for (float range = 100 ; range<1000 ; range=range*1.02)
  vStepCalculation (range) ;

vStepCalculation (123456789) ;
vStepCalculation (12345678.9) ;
vStepCalculation (1234567.89) ;
vStepCalculation (123456.789) ;
vStepCalculation (12345.6789) ;
vStepCalculation (1234.56789) ;
vStepCalculation (123.456789) ;
vStepCalculation (12.3456789) ; 
vStepCalculation (1.23456789) ;
vStepCalculation (0.123456789) ;
vStepCalculation (0.0123456789) ;
vStepCalculation (0.00123456789) ;
vStepCalculation (0.000123456789) ;
vStepCalculation (0.0000123456789) ;
vStepCalculation (0.00000123456789) ;
vStepCalculation (0.000000123456789) ; 
}

void loop() 
{
}

// ************************************************************************

int8_t vStepCalculation (float range)
{
//  Range= brute value  999 -> 99   100 -> 10
int8_t y ;
int8_t tenPower = 0 ;
Serial.print ("Range0 = ") ;
Serial.print (range) ;
while (range>1000)
{
range/=10;
tenPower++ ;  
Serial.print ("  R+ = ") ;
Serial.print (range) ;
}
while (range<100)
{
range*=10;
tenPower-- ;  
Serial.print ("  R- = ") ;
Serial.print (range) ;
}
// 100 < vStep < 999
Serial.print ("   \tRangeF = ") ;
Serial.print (range) ;
Serial.print (" \ttenPower = ") ;
Serial.println (tenPower) ;


uint8_t stepsY[] = {150,100,50,25,20,15,10,5} ;
float vStep ;
uint8_t nbHor ;
for (y =0 ; y<8 ; y++)
{
vStep = stepsY[y] ;
//nbHor = (int)(range/vStep) ;
nbHor = range/vStep ;
if (nbHor>3)
break;
}
range = range * pow (10, tenPower) ;  
vStep = vStep * pow (10, tenPower) ;  
nbHor++ ; // because n Vsteps >> n+1 horzontal lines !
Serial.print ("range = ") ;
Serial.print (range) ;
Serial.print ("  \tvStep = ") ;
Serial.print (vStep) ;
Serial.print (" \tNb lines = ") ;
Serial.print (nbHor) ;
Serial.print ("  ") ;
for (int x=0 ; x<nbHor ; x++)
   Serial.print ("#") ;
Serial.println ("\n") ;

return (vStep) ;
}

// ************************************************************************
